﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using AirFighter.DrawningObjects;
using AirFighter.MovementStrategy;
using AirFighter.Generics;
using System.IO;

namespace AirFighter.Generics
{
    internal class AirFighterGenericStorage
    {
        readonly Dictionary<string, AirFighterGenericCollection<DrawningAirFighter, DrawningObjectAirFighter>> _fighterStorages;

        public List<string> Keys => _fighterStorages.Keys.ToList();

        private readonly int _pictureWidth;
        private readonly int _pictureHeight;
        public AirFighterGenericStorage(int pictureWidth, int pictureHeight)
        {
            _fighterStorages = new Dictionary<string, AirFighterGenericCollection<DrawningAirFighter, DrawningObjectAirFighter>>();
            _pictureWidth = pictureWidth;
            _pictureHeight = pictureHeight;
        }
        public void AddSet(string name)
        {
            if (_fighterStorages.ContainsKey(name)) return;
            _fighterStorages[name] = new AirFighterGenericCollection<DrawningAirFighter, DrawningObjectAirFighter>(_pictureWidth, _pictureHeight);
        }

        public void DelSet(string name)
        {
            if (!_fighterStorages.ContainsKey(name)) return;
            _fighterStorages.Remove(name);
        }

        public AirFighterGenericCollection<DrawningAirFighter, DrawningObjectAirFighter>?
        this[string ind]
        {
            get
            {
                if (_fighterStorages.ContainsKey(ind)) return _fighterStorages[ind];
                return null;
            }
        }

        private static readonly char _separatorForKeyValue = '|';

        private readonly char _separatorRecords = ';';

        private static readonly char _separatorForObject = ':';
        public bool SaveData(string filename)
        {
            if (File.Exists(filename))
            {
                File.Delete(filename);
            }
            StringBuilder data = new();
            foreach (KeyValuePair<string, AirFighterGenericCollection<DrawningAirFighter, DrawningObjectAirFighter>> record in _fighterStorages)
            {
                StringBuilder records = new();
                foreach (DrawningAirFighter? elem in record.Value.GetAirFighter)
                {
                    records.Append($"{elem?.GetDataForSave(_separatorForObject)}{_separatorRecords}");
                }
                data.AppendLine($"{record.Key}{_separatorForKeyValue}{records}");
            }
            if (data.Length == 0)
            {
                return false;
            }
            using StreamWriter fs = new StreamWriter(filename);
            {
                fs.WriteLine($"AirFighterStorages{Environment.NewLine}");
                fs.WriteLine(data);
            }
            return true;
        }
        public bool LoadData(string filename)
        {
            if (!File.Exists(filename))
            {
                return false;
            }

            using (StreamReader fs = File.OpenText(filename))
            {

                string str = fs.ReadLine();

                if (str == null || str.Length == 0)
                {
                    return false;
                }

                if (!str.StartsWith("AirFighterStorages"))
                {
                    //если нет такой записи, то это не те данные
                    return false;
                }

                _fighterStorages.Clear();
                string strs = "";


                while ((strs = fs.ReadLine()) != null)
                {

                    if (strs == null)
                    {
                        return false;
                    }

                    string[] record = strs.Split(_separatorForKeyValue, StringSplitOptions.RemoveEmptyEntries);
                    if (record.Length != 2)
                    {
                        continue;
                    }
                    AirFighterGenericCollection<DrawningAirFighter, DrawningObjectAirFighter> collection = new(_pictureWidth, _pictureHeight);
                    string[] set = record[1].Split(_separatorRecords, StringSplitOptions.RemoveEmptyEntries);
                    foreach (string elem in set)
                    {
                        DrawningAirFighter? fighter = elem?.CreateDrawningAirFighter(_separatorForObject, _pictureWidth, _pictureHeight);
                        if (fighter != null)
                        {
                            if ((collection + fighter) == -1)
                            {
                                return false;
                            }
                        }
                    }
                    _fighterStorages.Add(record[0], collection);
                }
                return true;
            }
        }
    }
}
